package javax.swing.plaf.synth;

import javax.swing.DefaultCellEditor;
import javax.swing.JTextField;
import javax.swing.tree.DefaultTreeCellEditor;
import java.beans.PropertyChangeEvent;
import sun.swing.plaf.synth.SynthIcon;
import java.awt.Component;
import java.awt.Color;
import javax.swing.tree.TreeModel;
import java.util.Enumeration;
import java.awt.Insets;
import javax.swing.tree.TreePath;
import java.awt.Rectangle;
import java.awt.Graphics;
import javax.swing.plaf.UIResource;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeCellEditor;
import javax.swing.LookAndFeel;
import javax.swing.JTree;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.Icon;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicTreeUI;

public class SynthTreeUI extends BasicTreeUI implements PropertyChangeListener, SynthUI
{
    private SynthStyle style;
    private SynthStyle cellStyle;
    private SynthContext paintContext;
    private boolean drawHorizontalLines;
    private boolean drawVerticalLines;
    private Object linesStyle;
    private int padding;
    private boolean useTreeColors;
    private Icon expandedIconWrapper;
    
    public SynthTreeUI() {
        this.expandedIconWrapper = new SynthTreeUI.ExpandedIconWrapper();
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthTreeUI();
    }
    
    @Override
    public Icon getExpandedIcon() {
        return this.expandedIconWrapper;
    }
    
    @Override
    protected void installDefaults() {
        this.updateStyle(this.tree);
    }
    
    private void updateStyle(JTree c) {
        SynthContext context = this.getContext(c, 1);
        SynthStyle style = this.style;
        this.style = SynthLookAndFeel.updateStyle(context, this);
        if (this.style != style) {
            this.setExpandedIcon(this.style.getIcon(context, "Tree.expandedIcon"));
            this.setCollapsedIcon(this.style.getIcon(context, "Tree.collapsedIcon"));
            this.setLeftChildIndent(this.style.getInt(context, "Tree.leftChildIndent", 0));
            this.setRightChildIndent(this.style.getInt(context, "Tree.rightChildIndent", 0));
            this.drawHorizontalLines = this.style.getBoolean(context, "Tree.drawHorizontalLines", true);
            this.drawVerticalLines = this.style.getBoolean(context, "Tree.drawVerticalLines", true);
            this.linesStyle = this.style.get(context, "Tree.linesStyle");
            Object value = this.style.get(context, "Tree.rowHeight");
            if (value != null) {
                LookAndFeel.installProperty(c, "rowHeight", value);
            }
            Object value2 = this.style.get(context, "Tree.scrollsOnExpand");
            LookAndFeel.installProperty(c, "scrollsOnExpand", (value2 != null) ? value2 : Boolean.TRUE);
            this.padding = this.style.getInt(context, "Tree.padding", 0);
            this.largeModel = (c.isLargeModel() && c.getRowHeight() > 0);
            this.useTreeColors = this.style.getBoolean(context, "Tree.rendererUseTreeColors", true);
            LookAndFeel.installProperty(c, "showsRootHandles", Boolean.valueOf(this.style.getBoolean(context, "Tree.showsRootHandles", (boolean)Boolean.TRUE)));
            if (style != null) {
                this.uninstallKeyboardActions();
                this.installKeyboardActions();
            }
        }
        context.dispose();
        SynthContext context2 = this.getContext(c, Region.TREE_CELL, 1);
        this.cellStyle = SynthLookAndFeel.updateStyle(context2, this);
        context2.dispose();
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        this.tree.addPropertyChangeListener(this);
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, SynthLookAndFeel.getComponentState(component));
    }
    
    private SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    private SynthContext getContext(JComponent component, Region region) {
        return this.getContext(component, region, this.getComponentState(component, region));
    }
    
    private SynthContext getContext(JComponent component, Region region, int n) {
        return SynthContext.getContext(component, region, this.cellStyle, n);
    }
    
    private int getComponentState(JComponent component, Region region) {
        return 513;
    }
    
    @Override
    protected TreeCellEditor createDefaultCellEditor() {
        TreeCellRenderer cellRenderer = this.tree.getCellRenderer();
        if (cellRenderer != null && cellRenderer instanceof DefaultTreeCellRenderer) {
            return new SynthTreeUI.SynthTreeCellEditor(this.tree, (DefaultTreeCellRenderer)cellRenderer);
        }
        return new SynthTreeUI.SynthTreeCellEditor(this.tree, null);
    }
    
    @Override
    protected TreeCellRenderer createDefaultCellRenderer() {
        return new SynthTreeUI.SynthTreeCellRenderer();
    }
    
    @Override
    protected void uninstallDefaults() {
        SynthContext context = this.getContext(this.tree, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
        SynthContext context2 = this.getContext(this.tree, Region.TREE_CELL, 1);
        this.cellStyle.uninstallDefaults(context2);
        context2.dispose();
        this.cellStyle = null;
        if ((this.tree.getTransferHandler() instanceof UIResource)) {
            this.tree.setTransferHandler(null);
        }
    }
    
    @Override
    protected void uninstallListeners() {
        super.uninstallListeners();
        this.tree.removePropertyChangeListener(this);
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        context.getPainter().paintTreeBackground(context, graphics, 0, 0, component.getWidth(), component.getHeight());
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintTreeBorder(synthContext, graphics, n, n2, n3, n4);
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    protected void paint(SynthContext paintContext, Graphics graphics) {
        this.paintContext = paintContext;
        this.updateLeadSelectionRow();
        Rectangle clipBounds = graphics.getClipBounds();
        Insets insets = this.tree.getInsets();
        TreePath closestPathForLocation = this.getClosestPathForLocation(this.tree, 0, clipBounds.y);
        Enumeration<TreePath> visiblePaths = this.treeState.getVisiblePathsFrom(closestPathForLocation);
        int rowForPath = this.treeState.getRowForPath(closestPathForLocation);
        int n = clipBounds.y + clipBounds.height;
        TreeModel model = this.tree.getModel();
        SynthContext context = this.getContext(this.tree, Region.TREE_CELL);
        this.drawingCache.clear();
        this.setHashColor(paintContext.getStyle().getColor(paintContext, ColorType.FOREGROUND));
        if (visiblePaths != null) {
            int n2 = 0;
            Rectangle rectangle = new Rectangle(0, 0, this.tree.getWidth(), 0);
            TreeCellRenderer cellRenderer = this.tree.getCellRenderer();
            DefaultTreeCellRenderer defaultTreeCellRenderer = (cellRenderer instanceof DefaultTreeCellRenderer) ? ((DefaultTreeCellRenderer)cellRenderer) : null;
            this.configureRenderer(context);
            while (n2 == 0 && visiblePaths.hasMoreElements()) {
                TreePath treePath = (TreePath)visiblePaths.nextElement();
                Rectangle pathBounds = this.getPathBounds(this.tree, treePath);
                if (treePath != null && pathBounds != null) {
                    int leaf = model.isLeaf(treePath.getLastPathComponent());
                    int expandedState;
                    int hasBeenExpanded;
                    if (!(leaf == 0)) {
                        hasBeenExpanded = (expandedState = false);
                    }
                    else {
                        expandedState = this.treeState.getExpandedState(treePath);
                        hasBeenExpanded = this.tree.hasBeenExpanded(treePath);
                    }
                    rectangle.y = pathBounds.y;
                    rectangle.height = pathBounds.height;
                    this.paintRow(cellRenderer, defaultTreeCellRenderer, paintContext, context, graphics, clipBounds, insets, pathBounds, rectangle, treePath, rowForPath, (boolean)expandedState, (boolean)hasBeenExpanded, (boolean)leaf);
                    if (pathBounds.y + pathBounds.height >= n) {
                        n2 = 1;
                    }
                }
                else {
                    n2 = 1;
                }
                ++rowForPath;
            }
            int rootVisible = this.tree.isRootVisible();
            for (TreePath key = closestPathForLocation.getParentPath(); key != null; key = key.getParentPath()) {
                this.paintVerticalPartOfLeg(graphics, clipBounds, insets, key);
                this.drawingCache.put(key, Boolean.TRUE);
            }
            int n3 = 0;
            for (Enumeration visiblePaths2 = this.treeState.getVisiblePathsFrom(closestPathForLocation); n3 == 0 && visiblePaths2.hasMoreElements(); ++rowForPath) {
                TreePath path = (TreePath)visiblePaths2.nextElement();
                Rectangle pathBounds2 = this.getPathBounds(this.tree, path);
                if (path != null && pathBounds2 != null) {
                    int leaf2 = model.isLeaf(path.getLastPathComponent());
                    int expandedState2;
                    int hasBeenExpanded2;
                    if (!(leaf2 == 0)) {
                        hasBeenExpanded2 = (expandedState2 = false);
                    }
                    else {
                        expandedState2 = this.treeState.getExpandedState(path);
                        hasBeenExpanded2 = this.tree.hasBeenExpanded(path);
                    }
                    TreePath parentPath = path.getParentPath();
                    if (parentPath != null) {
                        if (this.drawingCache.get(parentPath) == null) {
                            this.paintVerticalPartOfLeg(graphics, clipBounds, insets, parentPath);
                            this.drawingCache.put(parentPath, Boolean.TRUE);
                        }
                        this.paintHorizontalPartOfLeg(graphics, clipBounds, insets, pathBounds2, path, rowForPath, (boolean)expandedState2, (boolean)hasBeenExpanded2, (boolean)leaf2);
                    }
                    else if (rootVisible && rowForPath == 0) {
                        this.paintHorizontalPartOfLeg(graphics, clipBounds, insets, pathBounds2, path, rowForPath, (boolean)expandedState2, (boolean)hasBeenExpanded2, (boolean)leaf2);
                    }
                    if (this.shouldPaintExpandControl(path, rowForPath, (boolean)expandedState2, (boolean)hasBeenExpanded2, (boolean)leaf2)) {
                        this.paintExpandControl(graphics, clipBounds, insets, pathBounds2, path, rowForPath, (boolean)expandedState2, (boolean)hasBeenExpanded2, (boolean)leaf2);
                    }
                    if (pathBounds2.y + pathBounds2.height >= n) {
                        n3 = 1;
                    }
                }
                else {
                    n3 = 1;
                }
            }
        }
        context.dispose();
        this.paintDropLine(graphics);
        this.rendererPane.removeAll();
        this.paintContext = null;
    }
    
    private void configureRenderer(SynthContext synthContext) {
        TreeCellRenderer cellRenderer = this.tree.getCellRenderer();
        if ((cellRenderer instanceof DefaultTreeCellRenderer)) {
            DefaultTreeCellRenderer defaultTreeCellRenderer = (DefaultTreeCellRenderer)cellRenderer;
            SynthStyle style = synthContext.getStyle();
            synthContext.setComponentState(513);
            Color textSelectionColor = defaultTreeCellRenderer.getTextSelectionColor();
            if (textSelectionColor == null || textSelectionColor instanceof UIResource) {
                defaultTreeCellRenderer.setTextSelectionColor(style.getColor(synthContext, ColorType.TEXT_FOREGROUND));
            }
            Color backgroundSelectionColor = defaultTreeCellRenderer.getBackgroundSelectionColor();
            if (backgroundSelectionColor == null || backgroundSelectionColor instanceof UIResource) {
                defaultTreeCellRenderer.setBackgroundSelectionColor(style.getColor(synthContext, ColorType.TEXT_BACKGROUND));
            }
            synthContext.setComponentState(1);
            Color textNonSelectionColor = defaultTreeCellRenderer.getTextNonSelectionColor();
            if (textNonSelectionColor == null || textNonSelectionColor instanceof UIResource) {
                defaultTreeCellRenderer.setTextNonSelectionColor(style.getColorForState(synthContext, ColorType.TEXT_FOREGROUND));
            }
            Color backgroundNonSelectionColor = defaultTreeCellRenderer.getBackgroundNonSelectionColor();
            if (backgroundNonSelectionColor == null || backgroundNonSelectionColor instanceof UIResource) {
                defaultTreeCellRenderer.setBackgroundNonSelectionColor(style.getColorForState(synthContext, ColorType.TEXT_BACKGROUND));
            }
        }
    }
    
    @Override
    protected void paintHorizontalPartOfLeg(Graphics g, Rectangle clipBounds, Insets insets, Rectangle bounds, TreePath path, int row, boolean isExpanded, boolean hasBeenExpanded, boolean isLeaf) {
        if (this.drawHorizontalLines) {
            super.paintHorizontalPartOfLeg(g, clipBounds, insets, bounds, path, row, isExpanded, hasBeenExpanded, isLeaf);
        }
    }
    
    @Override
    protected void paintHorizontalLine(Graphics graphics, JComponent component, int n, int n2, int n3) {
        this.paintContext.getStyle().getGraphicsUtils(this.paintContext).drawLine(this.paintContext, "Tree.horizontalLine", graphics, n2, n, n3, n, this.linesStyle);
    }
    
    @Override
    protected void paintVerticalPartOfLeg(Graphics g, Rectangle clipBounds, Insets insets, TreePath path) {
        if (this.drawVerticalLines) {
            super.paintVerticalPartOfLeg(g, clipBounds, insets, path);
        }
    }
    
    @Override
    protected void paintVerticalLine(Graphics graphics, JComponent component, int n, int n2, int n3) {
        this.paintContext.getStyle().getGraphicsUtils(this.paintContext).drawLine(this.paintContext, "Tree.verticalLine", graphics, n, n2, n, n3, this.linesStyle);
    }
    
    private void paintRow(TreeCellRenderer treeCellRenderer, DefaultTreeCellRenderer defaultTreeCellRenderer, SynthContext synthContext, SynthContext synthContext2, Graphics g, Rectangle rectangle, Insets insets, Rectangle rectangle2, Rectangle rectangle3, TreePath treePath, int row, boolean b, boolean b2, boolean b3) {
        int rowSelected = this.tree.isRowSelected(row);
        JTree.DropLocation dropLocation = this.tree.getDropLocation();
        int n = dropLocation != null && dropLocation.getChildIndex() == -1 && treePath == dropLocation.getPath();
        int componentState = 1;
        if (rowSelected || n) {
            componentState |= 0x200;
        }
        if (this.tree.isFocusOwner() && row == this.getLeadSelectionRow()) {
            componentState |= 0x100;
        }
        synthContext2.setComponentState(componentState);
        if (defaultTreeCellRenderer != null && defaultTreeCellRenderer.getBorderSelectionColor() instanceof UIResource) {
            defaultTreeCellRenderer.setBorderSelectionColor(this.style.getColor(synthContext2, ColorType.FOCUS));
        }
        SynthLookAndFeel.updateSubregion(synthContext2, g, rectangle3);
        synthContext2.getPainter().paintTreeCellBackground(synthContext2, g, rectangle3.x, rectangle3.y, rectangle3.width, rectangle3.height);
        synthContext2.getPainter().paintTreeCellBorder(synthContext2, g, rectangle3.x, rectangle3.y, rectangle3.width, rectangle3.height);
        if (this.editingComponent != null && this.editingRow == row) {
            return;
        }
        int leadSelectionRow;
        if (this.tree.hasFocus()) {
            leadSelectionRow = this.getLeadSelectionRow();
        }
        else {
            leadSelectionRow = -1;
        }
        this.rendererPane.paintComponent(g, treeCellRenderer.getTreeCellRendererComponent(this.tree, treePath.getLastPathComponent(), (boolean)rowSelected, b, b3, row, leadSelectionRow == row), this.tree, rectangle2.x, rectangle2.y, rectangle2.width, rectangle2.height, true);
        return;
    }
    
    private int findCenteredX(int n, int n2) {
        return this.tree.getComponentOrientation().isLeftToRight() ? (n - (int)Math.ceil((double)n2 / 2.0)) : (n - (int)Math.floor((double)n2 / 2.0));
    }
    
    @Override
    protected void paintExpandControl(Graphics g, Rectangle clipBounds, Insets insets, Rectangle bounds, TreePath path, int row, boolean isExpanded, boolean hasBeenExpanded, boolean isLeaf) {
        int pathSelected = this.tree.getSelectionModel().isPathSelected(path);
        int componentState = this.paintContext.getComponentState();
        if (!(pathSelected == 0)) {
            this.paintContext.setComponentState(componentState | 0x200);
        }
        super.paintExpandControl(g, clipBounds, insets, bounds, path, row, isExpanded, hasBeenExpanded, isLeaf);
        this.paintContext.setComponentState(componentState);
    }
    
    @Override
    protected void drawCentered(Component component, Graphics graphics, Icon icon, int n, int n2) {
        int iconWidth = SynthIcon.getIconWidth(icon, this.paintContext);
        int iconHeight = SynthIcon.getIconHeight(icon, this.paintContext);
        SynthIcon.paintIcon(icon, this.paintContext, graphics, this.findCenteredX(n, iconWidth), n2 - iconHeight / 2, iconWidth, iconHeight);
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
            this.updateStyle((JTree)propertyChangeEvent.getSource());
        }
        if ("dropLocation" == propertyChangeEvent.getPropertyName()) {
            this.repaintDropLocation((JTree.DropLocation)propertyChangeEvent.getOldValue());
            this.repaintDropLocation(this.tree.getDropLocation());
        }
    }
    
    @Override
    protected void paintDropLine(Graphics graphics) {
        JTree.DropLocation dropLocation = this.tree.getDropLocation();
        if (!this.isDropLine(dropLocation)) {
            return;
        }
        Color color = (Color)this.style.get(this.paintContext, "Tree.dropLineColor");
        if (color != null) {
            graphics.setColor(color);
            Rectangle dropLineRect = this.getDropLineRect(dropLocation);
            graphics.fillRect(dropLineRect.x, dropLineRect.y, dropLineRect.width, dropLineRect.height);
        }
        return;
    }
    
    private void repaintDropLocation(JTree.DropLocation dropLocation) {
        if (dropLocation == null) {
            return;
        }
        Rectangle r;
        if (this.isDropLine(dropLocation)) {
            r = this.getDropLineRect(dropLocation);
        }
        else {
            r = this.tree.getPathBounds(dropLocation.getPath());
            if (r != null) {
                r.x = 0;
                r.width = this.tree.getWidth();
            }
        }
        if (r != null) {
            this.tree.repaint(r);
        }
        return;
    }
    
    @Override
    protected int getRowX(int row, int depth) {
        return super.getRowX(row, depth) + this.padding;
    }
    
    static SynthContext access$300(SynthTreeUI synthTreeUI, JComponent component, Region region) {
        return synthTreeUI.getContext(component, region);
    }
    
    private class SynthTreeCellRenderer extends DefaultTreeCellRenderer implements UIResource
    {
        SynthTreeCellRenderer() {
        }
        
        @Override
        public String getName() {
            return "Tree.cellRenderer";
        }
        
        @Override
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
            if (!SynthTreeUI.this.useTreeColors && (sel || hasFocus)) {
                SynthLookAndFeel.setSelectedUI((SynthLabelUI)SynthLookAndFeel.getUIOfType(this.getUI(), SynthLabelUI.class), sel, hasFocus, tree.isEnabled(), false);
            }
            else {
                SynthLookAndFeel.resetSelectedUI();
            }
            return super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
        }
        
        @Override
        public void paint(Graphics g) {
            this.paintComponent(g);
            if (this.hasFocus) {
                SynthContext access$300 = SynthTreeUI.access$300(SynthTreeUI.this, SynthTreeUI.this.tree, Region.TREE_CELL);
                if (access$300.getStyle() == null) {
                    assert false : "SynthTreeCellRenderer is being used outside of UI that created it";
                    return;
                }
                else {
                    int n = 0;
                    Icon icon = this.getIcon();
                    if (icon != null && this.getText() != null) {
                        n = icon.getIconWidth() + Math.max(0, this.getIconTextGap() - 1);
                    }
                    if (this.selected) {
                        access$300.setComponentState(513);
                    }
                    else {
                        access$300.setComponentState(1);
                    }
                    if (this.getComponentOrientation().isLeftToRight()) {
                        access$300.getPainter().paintTreeCellFocus(access$300, g, n, 0, this.getWidth() - n, this.getHeight());
                    }
                    else {
                        access$300.getPainter().paintTreeCellFocus(access$300, g, 0, 0, this.getWidth() - n, this.getHeight());
                    }
                    access$300.dispose();
                }
            }
            SynthLookAndFeel.resetSelectedUI();
        }
    }
    
    private static class SynthTreeCellEditor extends DefaultTreeCellEditor
    {
        public SynthTreeCellEditor(JTree tree, DefaultTreeCellRenderer renderer) {
            super(tree, renderer);
            this.setBorderSelectionColor(null);
        }
        
        @Override
        protected TreeCellEditor createTreeCellEditor() {
            DefaultCellEditor defaultCellEditor = new DefaultCellEditor(new JTextField() {
                @Override
                public String getName() {
                    return "Tree.cellEditor";
                }
            });
            defaultCellEditor.setClickCountToStart(1);
            return defaultCellEditor;
        }
    }
    
    private class ExpandedIconWrapper extends SynthIcon
    {
        public void paintIcon(SynthContext context, Graphics graphics, int n, int n2, int n3, int n4) {
            if (context == null) {
                context = SynthTreeUI.this.getContext(SynthTreeUI.this.tree);
                SynthIcon.paintIcon(SynthTreeUI.this.expandedIcon, context, graphics, n, n2, n3, n4);
                context.dispose();
            }
            else {
                SynthIcon.paintIcon(SynthTreeUI.this.expandedIcon, context, graphics, n, n2, n3, n4);
            }
        }
        
        public int getIconWidth(SynthContext context) {
            int n;
            if (context == null) {
                context = SynthTreeUI.this.getContext(SynthTreeUI.this.tree);
                n = SynthIcon.getIconWidth(SynthTreeUI.this.expandedIcon, context);
                context.dispose();
            }
            else {
                n = SynthIcon.getIconWidth(SynthTreeUI.this.expandedIcon, context);
            }
            return n;
        }
        
        public int getIconHeight(SynthContext context) {
            int n;
            if (context == null) {
                context = SynthTreeUI.this.getContext(SynthTreeUI.this.tree);
                n = SynthIcon.getIconHeight(SynthTreeUI.this.expandedIcon, context);
                context.dispose();
            }
            else {
                n = SynthIcon.getIconHeight(SynthTreeUI.this.expandedIcon, context);
            }
            return n;
        }
    }
}
